c  Use  M-G EOS to determine density and K_s and G for a desired P and T
c  for a particular lower mantle mineralogy. Compare with PREM in the pressure
c  range 30Gpa to 120GPa. The mineralogical model is specified
c  by the foot temp of an adiabat at 660km and by the mole
c  fractions of MgPv, CaPv and Mw and by XFe and the partition
c  coefficient which specifies how much iron is in Pv versus
c  Mw.
c
c  The mantle adiabat is computed using the gruneisen parameter
c  for MgPv specified in the EOS adjusted so that rho0 is for
c  660 -- this means that rho0 is 4381 and gam0 is 1.35 and q is 1.4
c  -- to do this properly, should compute gamma for mineral
c  assemblage and then iterate to get a consistent mantle adiabat.
c  this probably isn't a serious problem since small changes in
c  gamma don't change the temperature much.
c
c  misfit calculations are done assuming .5% errors in rho and 
c  .1% errors in Vc. We calculate the maximum deviations as well
c  as chisq/N for rho and Vc separately. This is important as
c  rho and V_c can be systematically misfit in opposite directions
c  so changing temperature can not help in this case.
      implicit real*8(a-h,o-z)
      dimension pp(20),rp(20),rhop(20),vcp(20),vsp(20)
      data pi/3.1415926/
      rn=6371.
c*** get prem values interpolated to appropiate pressures
      open(9,file='prmtab2')
      do i=1,19
        read(9,*) pp(i),rp(i),rhop(i),vcp(i),vsp(i)
      enddo
      close(9)
c***

      gam0=1.35
      rho0=4381
      q=1.4
      print *,'enter adiabat foot temp (at 660)  : '
      read(*,*) tfoot
      print *,'enter Xfe and partition coefficient'
      read(*,*) xfe,fpc
      if(xfe.lt.0..or.xfe.gt.1.) then
         print *,'invalid input for xfe'
         stop
      end if
      if(fpc.lt.0.) then
         print *,'invalid input for partition coefficient'
         stop
      end if
      print *,'enter mole fractions of (mg,fe)-pv and ca-pv'
      read(*,*) xpv,xca
      if(xpv.lt.0..or.xca.lt.0..or.xpv+xca.gt.1.) then
         print *,'invalid input for xpv and/or xca'
         stop
      end if
      print *,'enter Qs of lower mantle -- 0 means no Q correction'
      read(*,*) qs
      facq=1
      if(qs.gt.0.) then
         beta=0.3
         arg=beta*pi*0.5
         ct=cos(arg)/sin(arg)
         facq=1.-0.5*ct/qs
      end if
      sum1=0.
      sum2=0.
      sum3=0.
      dev1=-1.e9
      dev2=-1.e9
      dev3=-1.e9
      
      n=19
c*** loop over pressure
      do i=1,n
        pobj=(25+i*5)*1.e9
c*** compute adiabat temp
        tk=tfoot*exp(gam0/q*(1.-(rho0/rhop(i))**q))
c*** get assemblage density and K_s and G for this P,T
        call assemb(tk,pobj,xfe,fpc,xpv,xca,
     +      rho,erho,vc,evc,vs,evs,vp,evp)
        vs=vs*facq
        rerr=rhop(i)*.005
        rerr=sqrt(rerr*rerr+erho*erho)
        rdev=(rho-rhop(i))/rerr
        sum1=sum1+rdev**2
        dev1=max(dev1,abs(rdev))
        verr=vcp(i)*.001
        verr=sqrt(verr*verr+evc*evc)
        vcdev=(vc-vcp(i))/verr
        sum2=sum2+vcdev**2
        dev2=max(dev2,abs(vcdev))
        verr=vsp(i)*.001
        verr=sqrt(verr*verr+evs*evs)
        vsdev=(vs-vsp(i))/verr
        sum3=sum3+vsdev**2
        dev3=max(dev3,abs(vsdev))
        write(66,999) tk,pobj*1.e-9,rhop(i),rho,rdev,vcp(i),
     +             vc,vcdev,vsp(i),vs,vsdev,erho,evc,evs
  999   format(14g14.6)
       qm=312
       qk=57823
       write(67,998) rp(i)*1000.,rho,vp,vs,qk,qm
  998  format(f10.1,3f10.3,2f10.1)
      enddo
c  these are chisq/n and max dev for fit to rho (1) and Vc (2) and Vs (3)
      chi1=sum1/n
      chi2=sum2/n
      chi3=sum3/n
      print *,chi1,chi2,chi3
      print *,dev1,dev2,dev3
      stop
      end

      subroutine assemb(tk,pobj,xfe,fpc,xpv,xca,
     +     rho,erho,vc,evc,vs,evs,vp,evp)
c*** routine to compute the density, bulk sound speed, Vs, and Vp of  a mineral assemblage
c*** consisting of (Mg,Fe)O, (Mg,Fe)SiO3, and CaSiO3 at temp tk (in K) and
c*** pressure pobj (in Pa)
c*** xpv is mole fraction of (Mg,Fe)Pv, xca is mole fraction of CaPv so
c*** 1.-xpv-xca is mole fraction of Mw.
c*** xfe is the fe/(fe+mg) ratio and fpc is the partition coefficient which
c*** governs how much iron goes into Pv and how much into Mw
c*** some values from Mattern et al (2005) are:
c*** pyrolite:  xpv=.622, xca=.051, xmw=.327, xfe=.104
c*** chondrite: xpv=.761, xca=.048, xmw=.191, xfe=.113
c*** solar:     xpv=.830, xca=.041, xmw=.129, xfe=.068
c***
c*** the partition coefficient (Mattern,eq 5) is around 0.25 if Fe preferentially
c*** goes into Mw and is 1 if Fe is evenly divided between Mw and Pv. The former
c*** seems to be the case if there is no Al in the lower mantle but the latter
c*** seems more likely with small amounts of Al (see Daniel, 2004)
c*** see sinmyo et al, 2008 for recent results on al-free samples for fe2+ 
c*** in which Kd increases from 0.2 to 0.3 from 30 to 120 GPa
c
c*** output is rho (in kg/m**3) and Vc, Vs, Vp (in m/s) along with error estimates.
c*** errors are estimated by redoing the calculation 100 times with thermoelastic parameters
c*** randomly taken from a uniform distribution spanning the experimental range 
c*** the Mie-Gruneisen EOS parameters and uncertainties are taken from Stixrude and 
c*** Lithgow-Bertelloni(2005) and are passed to the appropriate subroutines through common/pars. 
c*** See subroutine debye for a description of these parameters
      implicit real*8(a-h,o-z)
      parameter (ntry=100)
      dimension rhot(ntry),vct(ntry),vst(ntry),vpt(ntry)
c**** this common block holds all eos parameters -- see comments in subroutine debye
      common/pars/rho0,fk0,fkp0,gam0,q,tdeb,r3,tref,g0,gp0,es0
      real*4 uni,gasdev
c*** assume ref temp is 300K
      tref=300
      iseed=-2
      xmw=1.-xpv-xca
c*** figure out fe partitioning
      femg=xfe/(1.-xfe)
      xtot=xpv+xmw
      femgmw=femg*xtot/(fpc*xpv+xmw)
      femgpv=fpc*femgmw
      xfemw=femgmw/(1.+femgmw)
      xfepv=femgpv/(1.+femgpv)
c compute parameters ntry times to get estimate of errors. Function uni returns
c a random number between -1 and 1 with uniform probability, gasdev is guassian
      do jj=1,ntry
c*** note that rho0=(gmw/v0)*1000 in kg/m**3, v0 is in cm**3/mole, gmw is the molar weight in grams
c*** do magnesiowustite first
c*** MgO
      r3=1238
      gmw1=40.3044
      v0=11.244 +gasdev(iseed)*.01
      rho0=(gmw1/v0)*1000
      tdeb=773  +uni(iseed)*9
      gam0=1.50 +gasdev(iseed)*.02
      q=1.5     +uni(iseed)*.2
      fk0=161.e9+gasdev(iseed)*3.e9
      fkp0=3.9  +uni(iseed)*.2
      g0=130.e9 +gasdev(iseed)*3.e9
      gp0=2.2   +uni(iseed)*.1
      es0=2.3   +uni(iseed)*.2
      call getrho(tk,pobj,rho1,fks1,gam,cp,g1)
      v1=gmw1/rho1*1000
c*** FeO -- assumed stoiciometric
      r3=694
      gmw2=71.8464
      v0=12.062 +gasdev(iseed)*.01
      rho0=(gmw2/v0)*1000
      tdeb=455  +uni(iseed)*12
      gam0=1.28 +gasdev(iseed)*.11
      q=1.5     +uni(iseed)*1.0
      fk0=152.e9+gasdev(iseed)*1.e9
      fkp0=4.5  +uni(iseed)*.2
      g0=47.e9  +gasdev(iseed)*1.e9
      gp0=0.7   +uni(iseed)*.1
      es0=0.8   +uni(iseed)*.8
      call getrho(tk,pobj,rho2,fks2,gam,cp,g2)
      v2=gmw2/rho2*1000
c*** (Mg,Fe)O
c*** now combine mgo and feo -- assume linear in x
      cx=1.-xfemw
      vmw=v1*cx+v2*xfemw
      fksmw=fks1*cx+fks2*xfemw
      g_mw=g1*cx+g2*xfemw
      gmwmw=gmw1*cx+gmw2*xfemw
c*** now do mg,fe perovskite
c*** MgSiO3
      r3=1242
      gmw3=100.3837
      v0=24.445+gasdev(iseed)*.01
      rho0=(gmw3/v0)*1000
      fk0=251.e9+gasdev(iseed)*3.e9
c****  prob should be about 4
      fkp0=3.9+uni(iseed)*.2
      tdeb=912+uni(iseed)*100.
      gam0= 1.44+gasdev(iseed)*.05
      q=1.4+uni(iseed)*.3
      g0=175.e9+gasdev(iseed)*2.e9
c*** this value is in better agreement with Jackson's work on Al-bearing Pv
      gp0=1.72+uni(iseed)*.2
c*** this value is from Murakami et al, 2007 and doesn't appear to work well
c      g0=173.e9
c      gp0=1.56
      es0=2.6+uni(iseed)*.6
cc*** MgSiO3+5% mol Al2O3 -- from Matas et al, 2007
c      gmw3=100.4673
c      v0=24.58
c      fk0=249.e9
c      fkp0=4.0
c      rho0=(gmw3/v0)*1000
c      tdeb=1070
c      gam0= 1.48
c      q=1.4
c      r3=1241
c      g0=165e9
c      gp0=1.7
c      es0=2.6
      call getrho(tk,pobj,rho3,fks3,gam,cp,g3)
      v3=gmw3/rho3*1000
c*** FeSiO3
      r3=945
      gmw4=131.9307
      v0=25.40  +gasdev(iseed)*.05
      rho0=(gmw4/v0)*1000
      fk0=272.e9+gasdev(iseed)*20.e9
      fkp0=4.0  +uni(iseed)*1.0
      tdeb=841  +uni(iseed)*100.
      gam0=1.48 +gasdev(iseed)*.30
      q=1.4     +uni(iseed)*1.0
      g0=138.e9 +gasdev(iseed)*40.e9
      gp0=1.7   +uni(iseed)*.5
      es0=2.1   +uni(iseed)*1.0
      call getrho(tk,pobj,rho4,fks4,gam,cp,g4)
      v4=gmw4/rho4*1000
c*** (Mg,Fe)SiO3
c*** now combine mgsio3 and fesio3 -- assume linear in x
      cx=1.-xfepv
      vpv=v3*cx+v4*xfepv
      fkspv=fks3*cx+fks4*xfepv
      g_pv=g3*cx+g4*xfepv
      gmwpv=gmw3*cx+gmw4*xfepv
c*** CaSiO3
      r3=1074
      gmwca=116.1637
      v0=27.45   +gasdev(iseed)*.01
      rho0=(gmwca/v0)*1000
c*** these are from several authors (shim et al, 2000)
      fk0=240.e9 +gasdev(iseed)*15.e9
      fkp0=3.9   +uni(iseed)*.2
c*** these are from shim etal 2002 also caaracas et al, 2005 give 250, 4.1
c      fk0=255.e9+uni(iseed)*5.e9
c      fkp0=4.0+uni(iseed)*.2
      tdeb=984   +uni(iseed)*100.
      gam0=1.53  +gasdev(iseed)*.07
      q=1.6      +uni(iseed)*1.0
      g0=165.e9  +gasdev(iseed)*12.e9
      gp0=2.5    +uni(iseed)*.5
      es0=2.4    +uni(iseed)*1.0
c*** these are from Li et al, 2006 for v0,fk0,fkp0,gam0,q,g0,gp0
c      v0=27.44+uni(iseed)*.01
c      rho0=(gmwca/v0)*1000
c      fk0=237.e9+uni(iseed)*3.e9
c      fkp0=4.0+uni(iseed)*.1
c      q=1.0+uni(iseed)*0.3
c      g0=136.e9+uni(iseed)*12.e9
c      gp0=1.7+uni(iseed)*.3
c      es0=2.4+uni(iseed)*1.0
      call getrho(tk,pobj,rho5,fksca,gam,cp,g_ca)
      vca=gmwca/rho5*1000
c*** now combine using molar fractions
      vol=xmw*vmw+xpv*vpv+xca*vca
      gmw=xmw*gmwmw+xpv*gmwpv+xca*gmwca
      rho=(gmw/vol)*1000.
c*** do reuss average for bulk and shear modulus
      fg=vol/(xmw*vmw/g_mw+xpv*vpv/g_pv+xca*vca/g_ca)
      fks=vol/(xmw*vmw/fksmw+xpv*vpv/fkspv+xca*vca/fksca)
c*** now do voigt average
      fgv=(g_mw*(xmw*vmw) + g_pv*(xpv*vpv) + g_ca*(xca*vca))/vol
      fksv=(fksmw*(xmw*vmw) + fkspv*(xpv*vpv) + fksca*(xca*vca))/vol
c*** vrh
      fg=0.5*(fg+fgv)
      fks=0.5*(fks+fksv)
      rhot(jj)=rho
      vct(jj)=sqrt(fks/rho)
      vst(jj)=sqrt(fg/rho)
      vpt(jj)=sqrt((fks+4.*fg/3.)/rho)
c      print *,jj,rhot(jj),vct(jj),vst(jj),vpt(jj)
      enddo
c*** now compute medians and smads from ntry values
      call mean(rhot,ntry,arho,srho,rho,erho)
      call mean(vct,ntry,avc,svc,vc,evc)
      call mean(vst,ntry,avs,svs,vs,evs)
      call mean(vpt,ntry,avp,svp,vp,evp)
      print 777,pobj/1.e9,arho,srho,avc,svc,avs,svs,avp,svp
      print 777,pobj/1.e9,rho,erho,vc,evc,vs,evs,vp,evp
  777 format(9g14.6)
      erho=min(erho,srho)
      evc=min(evc,svc)
      evs=min(evs,svs)
      evp=min(evp,svp)
      return
      end

      subroutine mean(x,n,xm,xstd,xmed,smad)
c*** compute mean(xm), std(xstd), median(xmed) and smad(smad)
c***  of mean of vector x(1--n)
      implicit real*8(a-h,o-z)
      dimension x(*)
c*** mean
      sum=0.d0
      do i=1,n
        sum=sum+x(i)
      enddo
      xm=sum/n
c*** std
      sum=0.
      do i=1,n
         sum=sum+(x(i)-xm)**2
      enddo
      pts=n
      xstd=sqrt(sum/(pts-1))
c*** we don't want std on mean
c      spts=sqrt(pts)
c      xstd=xstd/spts
c median
      call shell(x,n)
      j = n / 2
      if ( (j*2).eq.n ) then
        xmed = ( x(n/2)+x(n/2+1) )/2.0
      else
        xmed = x((num/2) + 1)
      endif
c scaled median absolute devitaion -- note we overwrite x here
      do i=1,n
        x(i) = abs(xmed-x(i))
      enddo
      call shell(x,n)
      j = n / 2
      if ( (j*2).eq.n ) then
        errmed = ( x(n/2)+x(n/2+1) )/2.0
      else
        errmed = x((n/2) + 1)
      endif
      smad=errmed/0.6745
c*** we don'twant equiv of std on mean
c      smad=smad/spts
      return
      end

      subroutine shell(a,ncol)
c  shell sorts a(nc) into increasing order
      implicit real*8(a-h,o-z)
      dimension a(ncol)
      ig=ncol
    5 if(ig.le.1) return
      ig=ig/2
      im=ncol-ig
   10 iex=0
      do 20 i=1,im
      ipg=i+ig
      if(a(i).le.a(ipg)) go to 20
      te=a(i)
      a(i)=a(ipg)
      a(ipg)=te
      iex=iex+1
   20 continue
      if(iex.gt.0) go to 10
      go to 5
      end 


      subroutine getrho(tk,pobj,rho,fks,gam,cp,fg)
c*** routine to allow specification of pressure rather than volume and so
c*** return density for a particular P and T
c*** note that if the density could fall outside the range rho1--rho2, this
c*** routine could fail
      implicit real*8(a-h,o-z)
      common/pars/rho0,fk0,fkp0,gam0,q,tdeb,r3,tref,g0,gp0,es0
c*** define range of pressure possible -- note lower limit is not strictly correct but should be ok
      rho1=rho0
      p1=0.
      rho2=11000
      call debye(rho2,tk,p2,fks,gam,cp,fg)
c      print *,p1,pobj,p2
      if(pobj.gt.p1.and.pobj.lt.p2) then
c*** now interpolate to desired pressure
        call intrp(rho1,p1-pobj,rho2,p2-pobj,pobj,tk,rho,fks,gam,cp,fg)
c        print *,rho,fks,fg
      else
      print *,'limits exceeded in subroutine getrho'
        print *,p1,pobj,p2
        stop
      end if
      return
      end

      subroutine debye(rho,t,ptot,fks,gam,cp,fg)
c*** uses mie-gruneisen eos parameters to return pressure, K_s, gamma, cp and G
c*** for a desired density and temperature. EOS paramaeters are put in to 
c*** common/pars before calling -- for example, for mgo, we have
c      rho0=3586   : density at zero pressure and ref. temp in SI units
c      fk0=161.e9  : zerp pressure (and  ref temp) bulk modulus in Pa
c      fkp0=3.9    : dK/dP at zero pressure and ref temp
c      gam0= 1.50  : zero pressure value of Gruneisens ratio
c      q=1.5       : exponent in volume dependence of gamma
c      tdeb=773    : debye temp in Kelvin
c      r3=1238     : 3R in J/K/Kg (ie, converted from per mole) -- same as Dulong-Petit cv.
c      tref        : temperature of "cold part", usually about 300K
c
c  input: rho (desired density in SI) and t (desired temperature in K)
c  output: ptot (pressure in Pa), fks (K_s in Pa), gam (Gruneisen's ratio)
c          and cp (specific heat in J/K/Kg) and shear modulus (in Pa)
c****
      implicit real*8(a-h,o-z)
      common/pars/rho0,fk0,fkp0,gam0,q,tdeb,r3,tref,g0,gp0,es0
c      print 777,fk0,fkp0,tdeb,gam0,q,g0,gp0,es0
  777 format(8g14.6)
c*** calculate cold part using finite strain eos.
      call fseval(rho,pc,fkc,gc,gam,es,tdb,qq)
c*** pc is cold pressure, fkc is cold part of bulk modulus,gc is cold part of shear modulus
c*** compute cv
      xd=tdb/t
      xdr=tdb/tref
      cv=r3*3*tran04(xd)/xd**3
      cvr=r3*3*tran04(xdr)/xdr**3
c      print *,cv,cvr
c*** now compute the thermal pressure
      e=debye3(xd)*r3*t
      er=debye3(xdr)*r3*tref
      pth=gam*rho*(e-er)
      ptot=pc+pth
c*** now fix up bulk modulus and shear modulus for thermal part
c*** here we take into account cariable q
      fk=fkc+pth*(1.+gam-qq)-gam*gam*rho*(cv*t-cvr*tref)
      fg=gc-es*rho*(e-er)
c*** so alpha is now:
      alpha=gam*cv*rho/fk
c*** now do adiabatic bulk modulus and cp
      fks=fk*(1.+alpha*t*gam)
      cp=cv*(1.+alpha*t*gam)
      return
      end

      subroutine fseval(rho,pc,fk,g,gam,es,tdb,qq)
c*** evaluates finite strain eos for cold part of pressure, kt, and g
c*** note that all our eos are only 3'rd order so c3=0. below
c*** also computes the thermal parameters at the appropriate volume
      implicit real*8(a-h,o-z)
      common/pars/rho0,fk0,fkp0,gam0,q,tdeb,r3,tref,g0,gp0,es0
      c1=3*fk0
      c2=9*fk0*(4.-fkp0)
      c3=0.
      b1=c1/3.d0
      b2=(c2-7.d0*c1)/3.d0
      b3=(c3-9.d0*c2)/3.d0
      t2=2.d0/3.d0
      e=0.5d0*(1.-(rho/rho0)**t2)
      r=1.d0-2.d0*e
      e52=(dsqrt(r))**5
      pc=-e52*e*(c1+e*(c2/2.d0+e*c3/6.d0))
      fk=e52*(b1+e*(b2+0.5d0*b3*e))
c      fkp=5.d0/3.d0-r*e52*(b2+e*b3)/(3.d0*fk)
c*** here is computation of cold part of shear modulus from stixrude
      f=-e
      g=e52*(g0+f*(c1*gp0-5.d0*g0+
     +     f*(6.d0*fk0*gp0-24.d0*fk0-14.d0*g0+4.5d0*fk0*fkp0)))
c*** this is computation of gamma from stixrude
      h1=6.d0*gam0
      h2=h1*(h1-2.d0-3.d0*q)
      a1=-2.d0*gam0-2.d0*es0
      fac=1.d0+h1*f+0.5*h2*f*f
      fac2=0.5d0*r**2/fac
      gam=1.d0/6.d0*r*(h1+h2*f)/fac
      es=-gam-fac2*a1
c*** note that q is not a constant in the stixrude formulation
      cc=h2/h1
      qq=2.d0*gam-t2-cc*r/(3.d0*(1.d0+cc*f))
c*** debye temp has same volume dependence as mode frequencies
      tdb=tdeb*sqrt(fac)
      return
      end

      subroutine intrp(p1,x1,p2,x2,pobj,tk,b,fks,gam,cp,fmu)
c Given monotonic function x between p1 and p2, locates p for which x=0
      implicit real*8(a-h,o-z)
      data tol/1.d-6/
      b=p1
      fb=x1
      c=p2
      fc=x2
      if(fc*fb.ge.0.d0) return
      s=c
      fs=fc
c*** bisect ***
    5 h=0.5d0*(b+c)
c*** check for convergence ***
      t=dabs(h*tol)
      if(dabs(h-b).lt.t) return
      db=dabs(fb)
      dc=dabs(fc)
      if(db.le.dc) goto 10
      y=b
      fy=fb
      gg=b
      fg=fb 
      s=c
      fs=fc 
      goto 15
   10 y=s
      fy=fs 
      gg=c
      fg=fc 
      s=b
      fs=fb 
   15 if(fy.eq.fs) goto 20
      b=(s*fy-y*fs)/(fy-fs)
      if(dabs(b-s).lt.t) b=s+dsign(t,gg-s)
      if((b-h)*(s-b).lt.0.d0) b=h
      goto 25
   20 b=h
   25 continue
      call debye(b,tk,pcb,fks,gam,cp,fmu)
c      print *,b,pcb
      fb=pcb-pobj
      if(fg*fb.lt.0.d0) goto 30
      c=s
      fc=fs  
      goto 5
   30 c=gg
      fc=fg 
      goto 5     
      end

      function gasdev(idum)
c returns normally distributed noise with zero mean and unit variance
c set idum to a neg integer to initialize
      save
      data iset/0/
      if(iset.eq.0) then
   5    v1=2.*ran1(idum)-1.
        v2=2.*ran1(idum)-1.
        r=v1*v1+v2*v2
        if(r.ge.1.) goto 5
        fac=sqrt(-2.*log(r)/r)
        gset=v1*fac
        gasdev=v2*fac
        iset=1
      else
        gasdev=gset
        iset=0
      endif
c*** prevent rare extreme excursions
      if(gasdev.lt.-2.) gasdev=-2.
      if(gasdev.gt.2.) gasdev=2.
      return
      end

      function uni(idum)
c*** return random number with uniform prob between -1 and 1
      uni=2*ran1(idum)-1.
      return
      end

      function ran1(idum)
c returns random number with uniform prob between 0 and 1
c  set idum to any neg integer to initialize
      save
      common/rndom/r(97)
      parameter (m1=259200,ia1=7141,ic1=54773,rm1=1./m1)
      parameter (m2=134456,ia2=8121,ic2=28411,rm2=1./m2)
      parameter (m3=243000,ia3=4561,ic3=51349)
      data iff/0/
      if(idum.lt.0.or.iff.eq.0) then
        iff=1
        ix1=mod(ic1-idum,m1)
        ix1=mod(ia1*ix1+ic1,m1)
        ix2=mod(ix1,m2)
        ix1=mod(ia1*ix1+ic1,m1)
        ix3=mod(ix1,m3)
        do j=1,97
          ix1=mod(ia1*ix1+ic1,m1)
          ix2=mod(ia2*ix2+ic2,m2)
          r(j)=(float(ix1)+float(ix2)*rm2)*rm1
        enddo
        idum=1
      endif
      ix1=mod(ia1*ix1+ic1,m1)
      ix2=mod(ia2*ix2+ic2,m2)
      ix3=mod(ia3*ix3+ic3,m3)
      j=1+(97*ix3)/m3
      ran1=r(j)
      r(j)=(float(ix1)+float(ix2)*rm2)*rm1
      return
      end

      double precision function tran04(xvalue)
c
c    this program calculates the transport integral of order 4, defined as
c
c      tran04(x) = integral 0 to x { t**4 exp(t)/[exp(t)-1]**2 } dt
c
c  error returns:
c    if xvalue < 0.0, an error message is printed, and the program 
c    returns the value 0.0. 
c
c
c  machine-dependent constants:
c    nterms - integer - the number of terms of the array atran to be used.
c                       the recommended value is such that
c                             atran(nterms) < eps/100
c    xlow2 - double precision - the value below which tran04 = 0.0 to machine
c                   precision. the recommended value is
c                          cube root of (3*xmin)
c    xlow1 - double precision - the value below which tran04 = x**3/3 to
c                   machine precision. the recommended value is
c                             sqrt(8*epsneg)
c    xhigh1 - double precision - the value above which the exponential series for
c                    large x contains only one term. the recommended value
c                    is        - ln(eps).
c    xhigh2 - double precision - the value above which 
c                       tran04 = valinf  -  x**4 exp(-x)
c                    the recommended value is 4/eps
c    xhigh3 - double precision - the value of ln(epsneg). used to prevent overflow
c                    for large x.
c
c   other subroutines used:
c          cheval 
c
c     dr. allan j. macleod,
c     dept. of mathematics and statistics,
c     university of paisley ,
c     high st.,
c     paisley,
c     scotland.
c     pa1 2be.
c     (e-mail: macl_ms0@paisley.ac.uk )
c  latest revision:   23 january, 1996
c
      integer k1,k2,nterms,numexp,numjn
      double precision atran(0:19),cheval,eight,four,half,one,rk,
     &     rnumjn,sumexp,sum2,t,valinf,x,xhigh1,xhigh2,
     &     xhigh3,xk,xk1,xlow1,xlow2,xvalue,zero
      data zero,half,one/ 0.0 d 0 , 0.5 d 0 , 1.0 d 0 /
      data four,eight/ 4.0 d 0 , 8.0 d 0 /
      data numjn,rnumjn/ 4 , 4.0 d 0 / 
      data valinf/0.25975 75760 90673 16596 d 2/
      data atran/0.48075 70994 61511 05786  d    0,
     1          -0.81753 78810 32108 3956   d   -1,
     2           0.10027 00665 97516 2973   d   -1,
     3          -0.10599 33935 98201 507    d   -2,
     4           0.10345 06245 03040 53     d   -3,
     5          -0.96442 70548 58991        d   -5,
     6           0.87455 44408 5147         d   -6,
     7          -0.77932 12079 811          d   -7,
     8           0.68649 88614 10           d   -8,
     9          -0.59995 71076 4            d   -9,
     x           0.52136 62413              d  -10,
     1          -0.45118 3819               d  -11,
     2           0.38921 592                d  -12,
     3          -0.33493 60                 d  -13,
     4           0.28766 7                  d  -14,
     5          -0.24668                    d  -15,
     6           0.2113                     d  -16,
     7          -0.181                      d  -17,
     8           0.15                       d  -18,
     9          -0.1                        d  -19/
c  start execution
      x = xvalue
c  error test
      if ( x .lt. zero ) then
         print *,'error in tran04, argument less than zero'
         tran04 = zero
         return
      endif
c   set the machine-dependent constants.
      nterms=17
      xlow1=2.981d-08
      xlow2=6.838d-103
      xhigh1=36.044
      xhigh2=3.602d16
      xhigh3=-36.737
c   code for x < =  4.0
      if ( x .le. four ) then
         if ( x .lt. xlow2 ) then
            tran04 = zero
         else
            if ( x .lt. xlow1 ) then
               tran04 =  ( x ** ( numjn-1 ) ) / ( rnumjn - one ) 
            else
               t = ( ( ( x * x ) / eight ) - half ) - half
               tran04 = ( x ** ( numjn-1 ) ) * cheval(nterms,atran,t) 
            endif
         endif
      else
c  code for x > 4.0
         if ( x .gt. xhigh2 ) then
            sumexp = one
         else
            if ( x .le. xhigh1 ) then
               numexp = int ( xhigh1 / x ) + 1
               t = exp ( -x ) 
            else
               numexp = 1
               t = one
            endif
            rk = zero
            do k1 = 1 , numexp
               rk = rk + one
            enddo
            sumexp = zero
            do k1 = 1 , numexp
               sum2 = one
               xk = one/ ( rk * x ) 
               xk1 = one
               do k2 = 1 , numjn
                  sum2 = sum2 * xk1 * xk + one
                  xk1 = xk1 + one
               enddo
               sumexp = sumexp * t + sum2
               rk = rk - one
            enddo
         endif
         t = rnumjn * log( x ) - x + log( sumexp ) 
         if ( t .lt. xhigh3 ) then
            tran04 = valinf
         else
            tran04 = valinf - exp( t ) 
         endif
      endif
      return
      end

      double precision function debye3(xvalue)
c
c      this program calculates the debye function of order 3, defined as
c
c            debye3(x) = 3*[integral {0 to x} t^3/(exp(t)-1) dt] / (x^3)
c
c   error returns:
c      if xvalue < 0.0 an error message is printed and the
c      function returns the value 0.0
c
c   machine-dependent parameters:
c      nterms - integer - the no. of elements of the array adeb3.
c                         the recommended value is such that
c                             abs(adeb3(nterms)) < eps/100,
c                         subject to 1 <= nterms <= 18
c      xlow - double precision - the value below which
c                    debye3 = 1 - 3x/8 + x*x/20 to machine precision.
c                    the recommended value is 
c                        sqrt(8*epsneg)
c      xupper - double precision - the value above which 
c               debye3 = (18*zeta(4)/x^3) - 3*exp(-x)(x^3+3x^2+6x+6)/x^3.
c                      the recommended value is
c                          -log(2*eps)
c      xlim1 - double precision - the value above which debye3 = 18*zeta(4)/x^3
c                     the recommended value is 
c                          -log(xmin)
c      xlim2 - double precision - the value above which debye3 = 0.0 to machine
c                     precision. the recommended value is
c                          cube root(19/xmin)
c
c   other subroutines used:
c          cheval
c   author:
c          dr. allan j. macleod,
c          dept. of mathematics and statistics,
c          university of paisley
c          high st.
c          paisley
c          scotland
c          pa1 2be
c          (e-mail:  macl_ms0@paisley.ac.uk )
c   latest update:  23 january, 1996
c
      integer i,nexp,nterms
      double precision adeb3(0:18),cheval,debinf,eight,expmx,four,
     &     half,one,pt375,rk,sevp5,six,sum,t,three,twenty,x,
     &     xk,xki,xlim1,xlim2,xlow,xupper,xvalue,zero
      data zero,pt375/0.0 d 0 , 0.375 d 0/
      data half,one/0.5 d 0 , 1.0 d 0/
      data three,four,six/3.0 d 0 , 4.0 d 0 , 6.0 d 0/
      data sevp5,eight,twenty/7.5 d 0 , 8.0 d 0 , 20.0 d 0/
      data debinf/0.51329 91127 34216 75946 d -1/
      data adeb3/2.70773 70683 27440 94526  d    0,
     1           0.34006 81352 11091 75100  d    0,
     2          -0.12945 15018 44408 6863   d   -1,
     3           0.79637 55380 17381 64     d   -3,
     4          -0.54636 00095 90823 8      d   -4,
     5           0.39243 01959 88049        d   -5,
     6          -0.28940 32823 5386         d   -6,
     7           0.21731 76139 625          d   -7,
     8          -0.16542 09994 98           d   -8,
     9           0.12727 96189 2            d   -9,
     x          -0.98796 3459               d  -11,
     1           0.77250 740                d  -12,
     2          -0.60779 72                 d  -13,
     3           0.48075 9                  d  -14,
     4          -0.38204                    d  -15,
     5           0.3048                     d  -16,
     6          -0.244                      d  -17,
     7           0.20                       d  -18,
     8          -0.2                        d  -19/
c   start computation
      x = xvalue
c   error test
      if ( x .lt. zero ) then
         print *,'argument less than zero'
         debye3 = zero
         return
      endif
c   in the interests of speed, we set the machine-dependent constants (tgm).
      xlow=2.981d-8
      xupper=36.043
      xlim1=706.83
      xlim2=5.628d102
      nterms=16
c   code for x <= 4.0
      if ( x .le. four ) then
         if ( x .lt. xlow ) then
            debye3 = ( ( x - sevp5 ) * x + twenty ) / twenty
         else
            t = ( ( x * x / eight ) - half ) - half
            debye3 = cheval ( nterms , adeb3 , t ) - pt375 * x
         endif
      else
c   code for x > 4.0
         if ( x .gt. xlim2 ) then
            debye3 = zero
         else
            debye3 = one / ( debinf * x * x * x )
            if ( x .lt. xlim1 ) then
               expmx = exp ( -x )
               if ( x .gt. xupper ) then
                  sum = (((x+three)*x+six)*x+six) / (x*x*x)
               else
                  sum = zero
                  rk = aint ( xlim1 / x )
                  nexp = int ( rk )
                  xk = rk * x
                  do i = nexp,1,-1
                     xki = one / xk
                     t =  (((six*xki+six)*xki+three)*xki+one) / rk
                     sum = sum * expmx + t
                     rk = rk - one
                     xk = xk - x
                  enddo
               endif
               debye3 = debye3 - three * sum * expmx
            endif
         endif
      endif
      return
      end


      double precision function cheval(n,a,t)
c
c   this function evaluates a chebyshev series, using the
c   clenshaw method with reinsch modification, as analysed
c   in the paper by oliver.
c
c   input parameters
c       n - integer - the no. of terms in the sequence
c       a - double precision array, dimension 0 to n - the coefficients of
c           the chebyshev series
c       t - double precision - the value at which the series is to be
c           evaluated
c
c   references
c        "an error analysis of the modified clenshaw method for
c         evaluating chebyshev and fourier series" j. oliver,
c         j.i.m.a., vol. 20, 1977, pp379-391
c
c author:  dr. allan j. macleod,
c          dept. of mathematics and statistics,
c          university of paisley ,
c          high st.,
c          paisley,
c          scotland
c latest modification:   21 december , 1992
c
      integer i,n
      double precision a(0:n),d1,d2,half,t,test,tt,two,u0,u1,u2,zero
      data zero,half/ 0.0 d 0 , 0.5 d 0 /
      data test,two/ 0.6 d 0 , 2.0 d 0 /
      u1 = zero
c   if abs ( t )  < 0.6 use the standard clenshaw method
      if ( abs( t ) .lt. test ) then
         u0 = zero
         tt = t + t
         do i = n , 0 , -1
            u2 = u1
            u1 = u0
            u0 = tt * u1 + a( i ) - u2
         enddo
         cheval =  ( u0 - u2 ) / two
      else
c   if abs ( t )  > =  0.6 use the reinsch modification
         d1 = zero
c   t > =  0.6 code
         if ( t .gt. zero ) then
            tt =  ( t - half ) - half
            tt = tt + tt
            do  i = n , 0 , -1
               d2 = d1
               u2 = u1
               d1 = tt * u2 + a( i ) + d2
               u1 = d1 + u2
            enddo
            cheval =  ( d1 + d2 ) / two
         else
c   t < =  -0.6 code
            tt =  ( t + half ) + half
            tt = tt + tt
            do i = n , 0 , -1
               d2 = d1
               u2 = u1
               d1 = tt * u2 + a( i ) - d2
               u1 = d1 - u2
            enddo
            cheval =  ( d1 - d2 ) / two
         endif
      endif
      return
      end

